#include "x_alloc.hpp"
#include "x_crc32.hpp"

#define CRC32_POLYNOMIAL 0xedb88320

CRC32::CRC32() {
  init();
  buildTable();
}

CRC32::~CRC32() {
  if (table) free(table);
}

void CRC32::init() {
  crc = 0;
}

void CRC32::buildTable() {
  dword value;
  table = (dword*)smalloc(256 * sizeof(dword));
  for (int i = 0; i < 256; i++) {
    value = i;
    for (int j = 8; j > 0; j--) {
      if (value & 0x01)
	value = (value >> 1) ^ CRC32_POLYNOMIAL;
      else
	value >>= 1;
    }
    table[i] = value;
  }
}

void CRC32::addString(byte *p, dword n) {
  dword t1, t2;
  while (n--) {
    t1 = (crc >> 8) & 0xFFFFFF;
    t2 = table[((int) crc ^ *p++) & 0xff];
    crc = t1 ^ t2;
  }
}
